/*
** Filename: V2UDSC.c
**
** Automatically created by Application Wizard 1.4.2
**
** Part of solution Firmware in project V2UDSC
**
** Comments:
**
** Important: Sections between markers "FTDI:S*" and "FTDI:E*" will be overwritten by
** the Application Wizard
*/

#include "UART2DSC.h"

/* Default settings for UART interface */
#define DEF_UART_BAUD		UART_BAUD_9600
#define DEF_UART_DATA_BITS	UART_DATA_BITS_8
#define DEF_UART_FLOW		UART_FLOW_RTS_CTS
#define DEF_UART_STOP_BITS	UART_STOP_BITS_1
#define DEF_UART_PARITY		UART_PARITY_NONE

/* FTDI:STP Thread Prototypes */
vos_tcb_t *tcbSETUP;
vos_tcb_t *tcbUART;
vos_tcb_t *tcbBOMS;

void setup();
void UART();
void BOMS();
/* FTDI:ETP */

/* FTDI:SDH Driver Handles */
VOS_HANDLE hUSBHOST_1; // USB Host Port 1
VOS_HANDLE hUART; // UART Interface Driver
VOS_HANDLE hFAT_FILE_SYSTEM; // FAT File System for FAT32 and FAT16
VOS_HANDLE hBOMS; // Bulk Only Mass Storage for USB disks
/* FTDI:EDH */

vos_semaphore_t setupSem;
vos_semaphore_t dataSem;
vos_mutex_t mBufAccess;

FILE *file;
unsigned char leds;
unsigned char buf[512];
unsigned short pBuf = 0;

#define V2EVALBOARD_V1
#ifdef V2EVALBOARD_V1
unsigned char led0 = 0x02;
unsigned char led1 = 0x04;
unsigned char led2 = 0x20;
unsigned char led3 = 0x40;
unsigned char led4 = 0x00;
#else
unsigned char led0 = 0x08;
unsigned char led1 = 0x10;
unsigned char led2 = 0x20;
unsigned char led3 = 0x40;
unsigned char led4 = 0x00;
#endif

/* Declaration for IOMUx setup function */
void iomux_setup(void);
void monError();

/* Main code - entry point to firmware */
void main(void)
{
	/* FTDI:SDD Driver Declarations */
	// UART Driver configuration context
	uart_context_t uartContext;
	// USB Host configuration context
	usbhost_context_t usbhostContext;
	/* FTDI:EDD */

	/* FTDI:SKI Kernel Initialisation */
	vos_init(50, VOS_TICK_INTERVAL, VOS_NUMBER_DEVICES);
	vos_set_clock_frequency(VOS_48MHZ_CLOCK_FREQUENCY);
	vos_set_idle_thread_tcb_size(256);
	/* FTDI:EKI */

	iomux_setup();

	/* FTDI:SDI Driver Initialisation */
	// Initialise UART
	uartContext.buffer_size = VOS_BUFFER_SIZE_128_BYTES;
	uart_init(VOS_DEV_UART,&uartContext);

	// Initialise FAT File System Driver
	fatdrv_init(VOS_DEV_FAT_FILE_SYSTEM);

	// Initialise BOMS Device Driver
	boms_init(VOS_DEV_BOMS);

	// Initialise USB Host
	usbhostContext.if_count = 8;
	usbhostContext.ep_count = 16;
	usbhostContext.xfer_count = 2;
	usbhostContext.iso_xfer_count = 2;
	usbhost_init(VOS_DEV_USBHOST_1, -1, &usbhostContext);
	/* FTDI:EDI */

	/* FTDI:SCT Thread Creation */
	tcbSETUP = vos_create_thread_ex(25, 1024, setup, "Setup", 0);
	tcbUART = vos_create_thread_ex(24, 1024, UART, "Read", 0);
	tcbBOMS = vos_create_thread_ex(24, 1024, BOMS, "Write", 0);
	/* FTDI:ECT */

	vos_init_semaphore(&setupSem, 0);
	vos_init_semaphore(&dataSem, 0);
	vos_init_mutex(&mBufAccess, VOS_MUTEX_UNLOCKED);

	vos_start_scheduler();
}

/* FTDI:SSP Support Functions */

unsigned char usbhost_connect_state(VOS_HANDLE hUSB)
{
	unsigned char connectstate = PORT_STATE_DISCONNECTED;
	usbhost_ioctl_cb_t hc_iocb;

	if (hUSB)
	{
		hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_GET_CONNECT_STATE;
		hc_iocb.get        = &connectstate;
		vos_dev_ioctl(hUSB, &hc_iocb);

		// repeat if connected to see if we move to enumerated
		if (connectstate == PORT_STATE_CONNECTED)
		{
			vos_dev_ioctl(hUSB, &hc_iocb);
		}
	}
	return connectstate;
}


VOS_HANDLE fat_attach(VOS_HANDLE hMSI, unsigned char devFAT)
{
	fat_ioctl_cb_t           fat_ioctl;
	fatdrv_ioctl_cb_attach_t fat_att;
	VOS_HANDLE hFAT;

	// currently the MSI (BOMS or other) must be open
    // open the FAT driver
	hFAT = vos_dev_open(devFAT);

        // attach the FAT driver to the MSI driver
	fat_ioctl.ioctl_code = FAT_IOCTL_FS_ATTACH;
	fat_ioctl.set = &fat_att;
	fat_att.msi_handle = hMSI;
	fat_att.partition = 0;

	if (vos_dev_ioctl(hFAT, &fat_ioctl) != FAT_OK)
	{
                // unable to open the FAT driver
		vos_dev_close(hFAT);
		monError(); return NULL;
	}

	return hFAT;
}

void fat_detach(VOS_HANDLE hFAT)
{
	fat_ioctl_cb_t           fat_ioctl;

	if (hFAT)
	{
		fat_ioctl.ioctl_code = FAT_IOCTL_FS_DETACH;
		fat_ioctl.set = NULL;
		fat_ioctl.get = NULL;

		vos_dev_ioctl(hFAT, &fat_ioctl);
		vos_dev_close(hFAT);
	}
}


VOS_HANDLE boms_attach(VOS_HANDLE hUSB, unsigned char devBOMS)
{
	usbhost_device_handle ifDisk;
	usbhost_ioctl_cb_t hc_iocb;
	usbhost_ioctl_cb_class_t hc_iocb_class;
	msi_ioctl_cb_t boms_iocb;
	boms_ioctl_cb_attach_t boms_att;
	VOS_HANDLE hBOMS;

	// find BOMS class device
	hc_iocb_class.dev_class = USB_CLASS_MASS_STORAGE;
	hc_iocb_class.dev_subclass = USB_SUBCLASS_MASS_STORAGE_SCSI;
	hc_iocb_class.dev_protocol = USB_PROTOCOL_MASS_STORAGE_BOMS;

	// user ioctl to find first hub device
	hc_iocb.ioctl_code = VOS_IOCTL_USBHOST_DEVICE_FIND_HANDLE_BY_CLASS;
	hc_iocb.handle.dif = NULL;
	hc_iocb.set = &hc_iocb_class;
	hc_iocb.get = &ifDisk;

	if (vos_dev_ioctl(hUSB, &hc_iocb) != USBHOST_OK)
	{
		monError(); return NULL;
	}

	// now we have a device, intialise a BOMS driver with it
	hBOMS = vos_dev_open(devBOMS);

	// perform attach
	boms_att.hc_handle = hUSB;
	boms_att.ifDev = ifDisk;

	boms_iocb.ioctl_code = MSI_IOCTL_BOMS_ATTACH;
	boms_iocb.set = &boms_att;
	boms_iocb.get = NULL;

	if (vos_dev_ioctl(hBOMS, &boms_iocb) != MSI_OK)
	{
		vos_dev_close(hBOMS);
		hBOMS = NULL;
		monError(); return NULL;
	}

	return hBOMS;
}

void boms_detach(VOS_HANDLE hBOMS)
{
	msi_ioctl_cb_t boms_iocb;

	if (hBOMS)
	{
		boms_iocb.ioctl_code = MSI_IOCTL_BOMS_DETACH;
		boms_iocb.set = NULL;
		boms_iocb.get = NULL;

		vos_dev_ioctl(hBOMS, &boms_iocb);
		vos_dev_close(hBOMS);
	}
}

/* FTDI:ESP */

void open_drivers(void)
{
        /* Code for opening and closing drivers - move to required places in Application Threads */
        /* FTDI:SDA Driver Open */
        hUSBHOST_1 = vos_dev_open(VOS_DEV_USBHOST_1);
        hUART = vos_dev_open(VOS_DEV_UART);
        /* FTDI:EDA */
}

void monError()
{
	unsigned char leds;
	leds = led4;
	vos_gpio_write_port(GPIO_PORT_A, leds);
}

void setup()
{
	// Open a handle to all our devices...
	unsigned char status = 0;
	common_ioctl_cb_t uart_iocb;
	fat_context *fat_ctx = NULL;
	open_drivers();
	if (hUSBHOST_1 == NULL)
	{
		monError(); return;
	}

	// Set all pins to output.
	vos_gpio_set_port_mode(GPIO_PORT_A, 0xff);

	// Wait until enumeration is complete...
	while (usbhost_connect_state(hUSBHOST_1) != PORT_STATE_ENUMERATED)
	{
		// wait for enumeration to complete
		vos_delay_msecs(250);
		leds = led0;
		vos_gpio_write_port(GPIO_PORT_A, leds);
		vos_delay_msecs(250);
		leds = 0xFF;
		vos_gpio_write_port(GPIO_PORT_A, leds);
	}
	leds = 0x00;
	vos_gpio_write_port(GPIO_PORT_A, leds);

	hBOMS = boms_attach(hUSBHOST_1, VOS_DEV_BOMS);
	if (hBOMS == NULL)
	{
		monError();return;
	}

	hFAT_FILE_SYSTEM = fat_attach(hBOMS, VOS_DEV_FAT_FILE_SYSTEM);
	if (hFAT_FILE_SYSTEM == NULL)
	{
		monError();return;
	}

	// lastly attach the stdio file system to the FAT file system
	fsAttach(hFAT_FILE_SYSTEM);
	uart_iocb.ioctl_code = VOS_IOCTL_COMMON_ENABLE_DMA;
	uart_iocb.set.param = DMA_ACQUIRE_AS_REQUIRED;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set baud rate
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_BAUD_RATE;
	uart_iocb.set.uart_baud_rate = DEF_UART_BAUD;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set flow control
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_FLOW_CONTROL;
	uart_iocb.set.param = DEF_UART_FLOW;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set data bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_DATA_BITS;
	uart_iocb.set.param = DEF_UART_DATA_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set stop bits
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_STOP_BITS;
	uart_iocb.set.param = DEF_UART_STOP_BITS;
	vos_dev_ioctl(hUART, &uart_iocb);

	// UART set parity
	uart_iocb.ioctl_code = VOS_IOCTL_UART_SET_PARITY;
	uart_iocb.set.param = DEF_UART_PARITY;
	vos_dev_ioctl(hUART, &uart_iocb);
	vos_signal_semaphore(&setupSem);
}

void UART()
{
	unsigned short numRead;
	common_ioctl_cb_t uart_iocb;
	unsigned short dataAvail = 0;
	unsigned char testdata;

	vos_wait_semaphore(&setupSem);

	while(1)
	{
		// get bytes available...
		uart_iocb.ioctl_code = VOS_IOCTL_COMMON_GET_RX_QUEUE_STATUS;
		vos_dev_ioctl(hUART, &uart_iocb);
		dataAvail = uart_iocb.get.queue_stat; // How much data to read?

		if (dataAvail)
		{
			if (dataAvail > (sizeof(buf) - pBuf))
				dataAvail = (sizeof(buf) - pBuf);

			vos_lock_mutex(&mBufAccess);
			vos_dev_read(hUART, &buf[pBuf], dataAvail, &numRead);
			pBuf += numRead;
			vos_unlock_mutex(&mBufAccess);
			vos_signal_semaphore(&dataSem);
		}
	}
}

void BOMS()
{
	while(1)
	{
		vos_wait_semaphore(&dataSem);
		if (pBuf)
		{

			file = fopen("TEST.TXT", "a+");
			if (file)
			{
				vos_lock_mutex(&mBufAccess);
				fwrite(buf, (size_t)pBuf, sizeof(char), file);
				pBuf = 0;
				vos_unlock_mutex(&mBufAccess);
				fclose(file);
				file = NULL;
			}
			else
			{
				monError(); return; // System is knackered...
			}
		}
	}
}

